# --8<--8<--8<--8<--
#
# Copyright (C) 2006 Smithsonian Astrophysical Observatory
#
# This file is part of SAOsac::gi
#
# SAOsac::gi is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# SAOsac::gi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

package SAOsac::gi;

use strict;
use warnings;

use Carp;
use IO::File;

our $VERSION = '0.03';

our %id = (
	   zoff		=>  3,
	   ns		=>  4,
	   bundle_az	=>  7,
	   bundle_el	=>  8,
	   ri		=>  9,
	   ro		=> 10,
	   rings	=> 11,
	   spokes	=> 12,
	   src_dist	=> 13,
	   zoff_origin	=> 23,
	   
	   x0		=>  1,
	   y0		=>  2,
	   z0		=>  3,
	   p		=>  4,
	   k		=>  5,
	   rho0		=>  6,
	   az_mis	=>  7,
	   el_mis	=>  8,
	   l		=> 10,
	   eps_r	=> 14,
	   eps_i	=> 15,
	   dfm		=> 22,
	   optic_type	=> 29,
	   dfm_type	=> 30,
	  );

our %comment = (
		zoff		=> 'z offset of focal plane',
		ns		=> 'number of surfaces',
		bundle_az	=> 'incoming bundle azimuth angle',
		bundle_el	=> 'incoming bundle elevation angle',
		ri		=> 'inner radius of aperture annulus',
		ro		=> 'outer radius of aperture annulus',
		rings		=> 'number of rings',
		spokes		=> 'number of spokes',
		src_dist	=> 'source distance',
		zoff_origin	=> 'zoff measured wrt: ',
		
		x0		=> 'x coordinate of surface center',
		y0		=> 'y coordinate of surface center',
		z0		=> 'z coordinate of surface center',
		p		=> 'surface conic constant',
		k		=> 'surface factor',
		rho0		=> 'surface center radius',
		az_mis		=> 'surface misalignment: azimuth',
		el_mis		=> 'surface misalignment: elevation',
		l		=> 'surface length',
		eps_r		=> 'real part of dielectric constant',
		eps_i		=> 'imaginary part of dielectric constant',
		dfm		=> 'surface deformation: ',
		optic_type	=> 'optic type: ',
		dfm_type	=> 'deformation type: ',
	       );

our $fh;

# Preloaded methods go here.

sub create
{
  my ( $gi, $comment, $system, @surfaces ) = @_;

  # create .gi file
  $fh = IO::File->new($gi, 'w') or
    croak("couldn't create gi file `$gi'\n");

  print $fh $comment, "\n";

  gi(0, 'zoff',		sprintf("%.20g", $system->{'zoff'}));
  gi(0, 'zoff_origin',		$system->{'zoff_origin'},
				$system->{'zoff_origin'} ? 'STD' : 'GFOC');

  my $srfno = 0;

  shell( ++$srfno, $_ )
    foreach @surfaces;

  print $fh "-1,0,0\n";
  $fh->close;
}

sub shell
{
  my ( $shell, $info ) = @_;

  gi($shell, 'x0',	$info->{'x0'});
  gi($shell, 'y0',	$info->{'y0'});
  gi($shell, 'z0',	$info->{'z0'});
  gi($shell, 'p',	$info->{'p'});
  gi($shell, 'k',	$info->{'k'});
  gi($shell, 'rho0',	$info->{'rho0'});
  gi($shell, 'az_mis',	$info->{'az_mis'});
  gi($shell, 'el_mis',	$info->{'el_mis'});
  gi($shell, 'l',	$info->{'l'});

  gi($shell, 'eps_r',	$info->{'eps_r'}) if exists $info->{'eps_r'};

  gi($shell, 'eps_i',	$info->{'eps_i'}) if exists $info->{'eps_i'};

  gi($shell, 'dfm',	$info->{'dfm'}, $info->{'dfm'} ? 'yes' : 'no');

  gi($shell, 'dfm_type', $info->{'dfm_type'},
                         $info->{'dfm_type'} ? 'spline' : 'cogen');


  gi($shell, 'optic_type',	$info->{'optic_type'},
			$info->{'optic_type'} ? 'xray' : 'conventional');
}

sub gi
{
  my ( $rec_type, $param, $val, $c ) = @_;

  $c ||= '';

  printf $fh ( "%-32s%s\n", join(',', $rec_type, $id{$param}, $val), 
	       $comment{$param} . $c );
}

1;
